package com.pj.biz;

import java.util.List;

import com.pj.project.tb_gate_terminal.TbGateTerminal;
import com.pj.project.tb_gate_terminal.TbGateTerminalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_gate_terminal -- 车辆道闸
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/TbGateTerminal/")
public class TbGateTerminalController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbGateTerminalService tbGateTerminalService;

    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(TbGateTerminal.PERMISSION_CODE_ADD)
    public AjaxJson add(TbGateTerminal t) throws Exception{
        tbGateTerminalService.add(t);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbGateTerminal.PERMISSION_CODE_DEL)
    public AjaxJson delete(Long id) {
        tbGateTerminalService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    @SaCheckPermission(TbGateTerminal.PERMISSION_CODE_DEL)
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(TbGateTerminal.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(TbGateTerminal.PERMISSION_CODE_EDIT)
    public AjaxJson update(TbGateTerminal t) throws Exception{
        tbGateTerminalService.update(t);
        return AjaxJson.getSuccess();
    }

    @RequestMapping("check")
    public AjaxJson check(TbGateTerminal t) throws Exception{
        tbGateTerminalService.check(t);
        return AjaxJson.getSuccess();
    }
    @RequestMapping("open")
    @SaCheckPermission(TbGateTerminal.PERMISSION_CODE_OPEN)
    public AjaxJson open(Long id) throws Exception{
        tbGateTerminalService.open(id);
        return AjaxJson.getSuccess();
    }



    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Long id) {
        TbGateTerminal t = tbGateTerminalService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        List<TbGateTerminal> list = tbGateTerminalService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }


}
