package com.pj.project.tb_gate_terminal;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.MyConfig;
import com.pj.current.global.BusinessException;
import com.pj.current.satoken.StpUserUtil;
import com.pj.current.task.TaskService;
import com.pj.project.tb_car_black.TbCarBlack;
import com.pj.project.tb_car_black.TbCarBlackService;
import com.pj.project.tb_car_filing.TbCarFiling;
import com.pj.project.tb_car_filing.TbCarFilingService;
import com.pj.project.tb_car_visit_record.TbCarVisitRecord;
import com.pj.project.tb_car_visit_record.TbCarVisitRecordService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_gate_command_log.TbGateCommandLog;
import com.pj.project.tb_gate_command_log.TbGateCommandLogService;
import com.pj.project.tb_gate_terminal.bo.ReqBO;
import com.pj.project.tb_gate_terminal.task.GateTerminalBase64ToImage;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.utils.cache.RedisUtil;
import com.pj.utils.zkt.GateCommandUtils;
import com.pj.utils.zkt.ZKTecoUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;
import sun.net.util.IPAddressUtil;

import javax.annotation.Resource;

/**
 * Service: tb_gate_terminal -- 车辆道闸
 *
 * @author qzyReal
 */
@Service
@Transactional
@Slf4j
public class TbGateTerminalService extends ServiceImpl<TbGateTerminalMapper, TbGateTerminal> implements IService<TbGateTerminal> {


    @Resource
    private MyConfig myConfig;

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbGateTerminalMapper tbGateTerminalMapper;

    @Resource
    private TbVenuesService tbVenuesService;
    @Resource
    private TbGateCommandLogService tbGateCommandLogService;

    @Resource
    private TbCarVisitRecordService tbCarVisitRecordService;
    @Resource
    private TaskService taskService;

    @Resource
    private TbCarFilingService tbCarFilingService;

    @Resource
    private TbDeptService tbDeptService;
    @Resource
    @Lazy
    private TbCarBlackService tbCarBlackService;


    /**
     * 增
     */
    public void add(TbGateTerminal t) throws Exception {
        String ip = t.getIp();
        if (!IPAddressUtil.isIPv4LiteralAddress(ip)) {
            throw new BusinessException("IP地址不正确");
        }
        TbGateTerminal tbGateTerminal = findByIp(ip);
        if (tbGateTerminal != null) {
            throw new BusinessException("IP地址已被占用");
        }
        String sn = getSn(ip);
        Date now = new Date();
        String loginId = StpUtil.getLoginIdAsString();
        t.setSn(sn).setCreateTime(now).setUpdateTime(now).setCreateBy(loginId).setUpdateBy(loginId);
        this.save(t);
    }

    /**
     * 获取sn
     *
     * @param ip
     * @return
     * @throws Exception
     */
    private String getSn(String ip) throws Exception {
        Socket socket = new Socket();
        socket.connect(new InetSocketAddress(ip, myConfig.getGatePort()), 4000);
        String result = ZKTecoUtils.sendAndGetResult(socket, GateCommandUtils.getSn());
        return JSONUtil.parseObj(result).getStr("value");
    }

    private TbGateTerminal findByIp(String ip) {
        QueryWrapper<TbGateTerminal> ew = new QueryWrapper<>();
        ew.eq("ip", ip);
        return getOne(ew);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbGateTerminal t) throws Exception {
        TbGateTerminal ipTerminal = findByIp(t.getIp());
        if (!t.getId().equals(ipTerminal.getId())) {
            throw new BusinessException("IP地址已被占用");
        }
        TbGateTerminal db = this.getById(t.getId());
        if (!StrUtil.equals(t.getIp(), db.getIp())) {
            String sn = getSn(t.getIp());
            t.setSn(sn);
        }
        t.setUpdateBy(StpUtil.getLoginIdAsString()).setUpdateTime(new Date());
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbGateTerminal getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbGateTerminal> getList(SoMap so) {

        return tbGateTerminalMapper.getList(so);
    }

    /**
     * 检测通讯
     *
     * @param t
     * @throws Exception
     */
    public void check(TbGateTerminal t) throws Exception {
        String ip = t.getIp();
        getSn(ip);
    }

    /**
     * 远程开门
     *
     * @param id
     */
    public void open(Long id) throws Exception {
        TbGateTerminal tbGateTerminal = this.getById(id);
        TbVenues tbVenues = tbVenuesService.getById(tbGateTerminal.getVenuesId());
        log.info("开启道闸========:{}", tbVenues.getName());
        Socket socket = new Socket();
        socket.connect(new InetSocketAddress(tbGateTerminal.getIp(), myConfig.getGatePort()), 4000);
        String result = ZKTecoUtils.sendAndGetResult(socket, GateCommandUtils.openCommand());
        Integer code = JSONUtil.parseObj(result).getInt("state_code");
        if (code == null || code != 200) {
            throw new BusinessException("开闸失败");
        }
        TbGateCommandLog log = new TbGateCommandLog();
        log.setTerminalName(tbGateTerminal.getName()).setVenuesName(tbVenues.getName())
                .setCommand(GateCommandUtils.openCommand()).setResponseContent(result).setSn(tbGateTerminal.getSn())
                .setCreateBy(StpUserUtil.getAdminName()).setCreateTime(new Date());
        tbGateCommandLogService.save(log);
    }

    private static TimedCache<Long, String> RECORD_CACHE = CacheUtil.newTimedCache(10000);

    public void record(ReqBO bo) {
        String ip = bo.getIpaddr();
        TbGateTerminal tbGateTerminal = this.findByIp(ip);
        ReqBO.ResultBean.PlateResultBean plateResultBean = bo.getResult().getPlateResult();
        String carNo = plateResultBean.getLicense();
        if (StrUtil.contains(carNo, "无")) {
            log.info("车牌识别错误=======无法处理:{}", carNo);
            return;
        }
        if (tbGateTerminal == null) {
            log.error("===========非法设备============");
            return;
        }
        Long terminalId = tbGateTerminal.getId();
        String cacheCarNo = RECORD_CACHE.get(terminalId);
        if (StrUtil.isNotEmpty(cacheCarNo)) {
            log.error("===========40秒内重复识别:{}============", carNo);
            return;
        }
        if (tbGateTerminal.getDirection().equals(2)) {
            handlerOut(tbGateTerminal, bo);
            return;
        } else {
            handlerIn(tbGateTerminal, bo);
        }
        RECORD_CACHE.put(tbGateTerminal.getId(), carNo);
        tbGateTerminal.setLastOnlineTime(new Date());
        this.updateById(tbGateTerminal);
    }

    /**
     * 入场===>缓存是否开闸
     *
     * @param tbGateTerminal
     * @param bo
     */
    private void handlerIn(TbGateTerminal tbGateTerminal, ReqBO bo) {
        ReqBO.ResultBean.PlateResultBean plateResultBean = bo.getResult().getPlateResult();
        String base64Image = plateResultBean.getImageFragmentFile();
        String carNo = plateResultBean.getLicense();
        String now = DateUtil.now();
        TbCarVisitRecord carVisitRecord=  saveRecord(carNo, base64Image, tbGateTerminal, 1, 0);
        cacheResult(carNo, now, tbGateTerminal.getVenuesId(), tbGateTerminal.getId(),carVisitRecord.getId());
    }

    /**
     * 离场车辆 直接放行
     *
     * @param tbGateTerminal
     */
    private void handlerOut(TbGateTerminal tbGateTerminal, ReqBO bo) {
        ReqBO.ResultBean.PlateResultBean plateResultBean = bo.getResult().getPlateResult();
        String base64Image = plateResultBean.getImageFragmentFile();
        String carNo = plateResultBean.getLicense();
        log.info("车辆离场:{}", carNo);
        if (StrUtil.contains(carNo, "无")) {
            log.info("车牌识别错误=======无法处理:{}", carNo);
            return;
        }
        saveRecord(carNo, base64Image, tbGateTerminal, 2, 1);
        try {
            this.open(tbGateTerminal.getId());
        } catch (Exception e) {
            log.error("抬杆失败:{}", e.getMessage());
        }
    }

    /**
     * 保存记录
     *
     * @param carNo
     * @param base64Image
     * @param tbGateTerminal
     * @param direction
     */
    private TbCarVisitRecord saveRecord(String carNo, String base64Image, TbGateTerminal tbGateTerminal, Integer direction, int allowPass) {
        Long channelId = tbGateTerminal.getVenuesId();
        Date now = new Date();
        TbVenues tbVenues = tbVenuesService.getById(channelId);
        TbCarVisitRecord tbCarVisitRecord = new TbCarVisitRecord();
        tbCarVisitRecord.setCarNo(carNo).setAllowPass(allowPass)
                .setChannelId(tbGateTerminal.getVenuesId())
                .setChannel(tbVenues.getName()).setDirection(direction)
                .setVisitTime(now).setUpdateTime(now).setCreateTime(now).setRecordType(2);
        TbCarFiling carFiling = tbCarFilingService.findByCarNo(carNo);
        if (carFiling != null) {
            TbDept tbDept = tbDeptService.getById(carFiling.getDeptId());
            tbCarVisitRecord.setDriverName(carFiling.getDriverName()).setDeptId(carFiling.getDeptId()).setDeptName(tbDept.getName())
                    .setDriverIdCard(carFiling.getDriverIdCard()).setType(carFiling.getType())
                    .setDriverContact(carFiling.getDriverContact());
        }
        tbCarVisitRecordService.save(tbCarVisitRecord);
        taskService.addTask(new GateTerminalBase64ToImage(RandomUtil.randomNumbers(5), 1423, tbCarVisitRecord.getId(), base64Image));
        return tbCarVisitRecord;
    }


    private void cacheResult(String carNo, String now, Long venuesId, Long terminalId,Long recordId) {
        TbCarBlack tbCarBlack = tbCarBlackService.findByCarNo(carNo, now);
        //是否在黑名单上
        if (tbCarBlack != null) {
            log.error("黑名单车辆:{}", carNo);
            return;
        }
        TbCarFiling tbCarFiling = tbCarFilingService.findByCarNo(carNo);
        //是否备案
        if (tbCarFiling == null) {
            log.error("车辆未备案:{}", carNo);
            return;
        }
        //备案车，如果属于的组织只需要满足一个条件的直接放行
        Long deptId = tbCarFiling.getDeptId();
        TbDept tbDept = tbDeptService.getById(deptId);
        if (TbDept.PassTypeEnum.ONE.getType().equals(tbDept.getPassType())) {
            log.error("组织满足条件二，直接放行:{}", carNo);
            try {
                open(terminalId);
                tbCarVisitRecordService.updatePass(recordId);
            } catch (Exception e) {
                log.error("开闸失败:{}", e.getMessage());
            }
        }
        RedisUtil.set("result:" + venuesId, terminalId + "");
        RedisUtil.set("record:" + venuesId, recordId + "");
    }

    public List<TbGateTerminal> findByVenuesId(Long venuesId) {
        QueryWrapper<TbGateTerminal> ew = new QueryWrapper<>();
        ew.eq("venues_id", venuesId);
        return list(ew);
    }
}
